// Radar program for Arduino

#include <SoftwareServo.h>
#include <NewSoftSerial.h>

// Digital pins, to whom you have connected the sensor
#define SENSOR_PIN_TX 2
#define SENSOR_PIN_RX 3

// Digital pin, to whom you have connected PWM controlling signal
#define SERVO_PIN 4

// Variable containing present sensor direction
int direction = 90;
// Variable telling if distance value was requested
byte read_distance = 0;

// Declaration of serial sensor communication object
NewSoftSerial sensor_serial(SENSOR_PIN_TX, SENSOR_PIN_RX, true);
// Declaration of servo controlling object
SoftwareServo sensor_servo;

// Function returning distance in inches
int get_distance()
{
  char data[4] = "000";
  char header = ' ';
  while (header != 'R')
    header = sensor_serial.read();
  while (sensor_serial.available() < 4);
  for (byte i = 0; i < 3; i++)
    data[i] = sensor_serial.read();
  header = sensor_serial.read();
  if (header != 13)
    return -1;
  return atoi(data);
}

// Function sending number in "nnn" (3-digit) format
void send_number(int number)
{
  if (number < 100)
    Serial.print("0");
  if (number < 10)
    Serial.print("0");
  Serial.print(number, DEC);
}

// Function sending distance data to the computer
void send_data(int distance)
{
  Serial.print("D");
  send_number(distance);
  Serial.println("");
}

// Function receiving commands from the computer
// Argument: timeout - time in [ms] after which the attempts to read data are stopped
int get_direction(unsigned long timeout=1000)
{
  char data[4] = "000";
  char header = ' ';
  // Present time storing
  unsigned long time = millis();
  // Header downloading
  header = Serial.read();
  // Sprawdzenie czy jest to żadanie odległości
  if (header == 'D')
  {
    read_distance = 1;
    return -4;
  }
  // Checking, if it's distance request
  if (header != 'R')
    return -1; // Jeśli inny to zwrócenie błędu o kodzie -1
  // Oczekiwanie na pojawienie się 3 bajtów danych
  while (Serial.available() < 4)
  {
    // Checking, if direction command header is present
    if ((millis() - time) > timeout)
      return -2; // Jeśli tak to zwrócenie kodu błędu -2
  }
  // Distance digits reading
  for (byte i = 0; i < 3; i++)
    data[i] = Serial.read();
  // Checking for proper data transmission ending
  header = Serial.read();
  if (header != '\n')
    return -3; // "-3" value means error
  return atoi(data);
}

// Function checks if rotation command was sent and executes it
void rotate()
{
  // Checking, if there are any data in the buffer
  while (Serial.available())
  {
    direction = get_direction();
    if (direction >= 0)
      // If the data was correct, setting servo angle
      sensor_servo.write(direction);
  }
}

// Clearing the software communication buffer
void clear_buffer()
{
  while (sensor_serial.available())
    sensor_serial.flush();
}

// Function returns distance, if it was requested
void distance()
{
  // Checking, if distance was requested
  if (read_distance == 1)
  {
    // Sensor activation
    digitalWrite(SENSOR_PIN_RX, HIGH);
    // 20 us for the sensor to distance measurement
    delayMicroseconds(20);
    // Waiting for data from sensor
    while (sensor_serial.available())
    {
      int distance = get_distance();
      if (distance > 0)
      {
        // If the sensor returned proper data, sending it to the computer
        send_data(distance);
        // Deleting distance request
        read_distance = 0;
        // Stopping the sensor
        digitalWrite(SENSOR_PIN_RX, LOW);
        // Buffer clearing
        clear_buffer();
      }
    }
  }
}


void setup()
{
  Serial.begin(9600);
  pinMode(SENSOR_PIN_TX, INPUT);
  pinMode(SENSOR_PIN_RX, OUTPUT);
  digitalWrite(SENSOR_PIN_RX, LOW);
  sensor_serial.begin(9600);
  // Servo callibration - minimum and maximum impulse length in µs
  sensor_servo.setMinimumPulse(560);
  sensor_servo.setMaximumPulse(2550);
  // Setting servo control pin
  sensor_servo.attach(SERVO_PIN);
  // Aligning the servo to the center
  sensor_servo.write(90);
  // Waiting for servo to align and turning the sensor on
  delay(1000);
  // Buffer cleaning
  clear_buffer();
}


void loop()
{
  rotate();
  distance();
  
  // Sending impulse to servo
  SoftwareServo::refresh();
  delay(10);
}
